@@ -11,12 +11,14 @@ import (
 	"io/ioutil"
 	"log"
 	"os"
+	"path"
 	"path/filepath"
 	"regexp"
 	"sort"
 	"strconv"
 	"strings"
 
+	"github.com/chzyer/readline"
 	"github.com/multiprocessio/datastation/runner"
 
 	"github.com/google/uuid"
@@ -96,6 +98,12 @@ func dumpJSONFile(file string, pretty bool, schema bool) error {
 	}
 	defer fd.Close()
 
+	size := int64(-1)
+	fi, err := fd.Stat()
+	if err == nil {
+		size = fi.Size()
+	}
+
 	if schema {
 		s, err := runner.ShapeFromFile(file, "doesn't-matter", runner.DefaultShapeMaxBytesToRead, 100)
 		if err != nil {
@@ -123,48 +131,57 @@ func dumpJSONFile(file string, pretty bool, schema bool) error {
 		return nil
 	}
 
-	s, err := runner.ShapeFromFile(file, "doesn't-matter", runner.DefaultShapeMaxBytesToRead, 100)
-	if err != nil {
-		return err
-	}
-	var columns []string
-	for name := range s.ArrayShape.Children.ObjectShape.Children {
-		columns = append(columns, name)
-	}
-	sort.Strings(columns)
-
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetHeader(columns)
-	table.SetAutoFormatHeaders(false)
-
-	dec := json.NewDecoder(fd)
 	var rows []map[string]interface{}
-	err = dec.Decode(&rows)
-	if err != nil {
-		return err
-	}
-
-	for _, objRow := range rows {
-		var row []string
-		for _, column := range columns {
-			var cell string
-			switch t := objRow[column].(type) {
-			case bool, byte, complex64, complex128, error, float32, float64,
-				int, int8, int16, int32, int64,
-				uint, uint16, uint32, uint64, uintptr:
-				cell = fmt.Sprintf("%#v", t)
-			case string:
-				cell = t
-			default:
-				cellBytes, _ := json.Marshal(t)
-				cell = string(cellBytes)
-			}
-			row = append(row, cell)
+	if size != 0 {
+		s, err := runner.ShapeFromFile(file, "doesn't-matter", runner.DefaultShapeMaxBytesToRead, 100)
+		if err != nil {
+			return err
 		}
-		table.Append(row)
+		var columns []string
+		for name := range s.ArrayShape.Children.ObjectShape.Children {
+			columns = append(columns, name)
+		}
+		sort.Strings(columns)
+
+		table := tablewriter.NewWriter(os.Stdout)
+		table.SetHeader(columns)
+		table.SetAutoFormatHeaders(false)
+
+		dec := json.NewDecoder(fd)
+		err = dec.Decode(&rows)
+		if err != nil {
+			return err
+		}
+
+		for _, objRow := range rows {
+			var row []string
+			for _, column := range columns {
+				var cell string
+				switch t := objRow[column].(type) {
+				case bool, byte, complex64, complex128, error, float32, float64,
+					int, int8, int16, int32, int64,
+					uint, uint16, uint32, uint64, uintptr:
+					cell = fmt.Sprintf("%#v", t)
+				case string:
+					cell = t
+				default:
+					cellBytes, _ := json.Marshal(t)
+					cell = string(cellBytes)
+				}
+				row = append(row, cell)
+			}
+			table.Append(row)
+		}
+
+		table.Render()
+	}
+
+	if len(rows) == 1 {
+		fmt.Println("(1 row)")
+	} else {
+		fmt.Printf("(%d rows)\n", len(rows))
 	}
 
-	table.Render()
 	return nil
 }
 
@@ -219,6 +236,96 @@ func importFile(projectId string, file, mimetype string, ec runner.EvalContext)
 	}, nil
 }
 
+func runQuery(queryRaw string, project *runner.ProjectState, ec *runner.EvalContext, args *args, files []string) error {
+	query := rewriteQuery(queryRaw)
+	panel := &runner.PanelInfo{
+		Type:    runner.DatabasePanel,
+		Content: query,
+		Id:      uuid.New().String(),
+		Name:    uuid.New().String(),
+		DatabasePanelInfo: &runner.DatabasePanelInfo{
+			Database: runner.DatabasePanelInfoDatabase{
+				ConnectorId: project.Connectors[0].Id,
+			},
+		},
+	}
+
+	err := ec.EvalDatabasePanel(project, 0, panel, nil, args.cacheSettings)
+	if err != nil {
+		if e, ok := err.(*runner.DSError); ok && e.Name == "NotAnArrayOfObjectsError" {
+			rest := "."
+			nth, err := strconv.Atoi(e.TargetPanelId)
+			if err == nil {
+				rest = ": " + files[nth] + "."
+			}
+			return fmt.Errorf("Input is not an array of objects%s\n", rest)
+		}
+	}
+
+	resultFile := ec.GetPanelResultsFile(project.Id, panel.Id)
+	return dumpJSONFile(resultFile, args.pretty, args.schema)
+}
+
+func repl(project *runner.ProjectState, ec *runner.EvalContext, args *args, files []string) error {
+	completer := readline.NewPrefixCompleter(
+		readline.PcItem("SELECT"),
+		readline.PcItem("FROM"),
+		readline.PcItem("WHERE"),
+		readline.PcItem("AND"),
+		readline.PcItem("OR"),
+		readline.PcItem("IN"),
+		readline.PcItem("JOIN"),
+	)
+
+	filterInput := func(r rune) (rune, bool) {
+		switch r {
+		// block CtrlZ feature
+		case readline.CharCtrlZ:
+			return r, false
+		}
+		return r, true
+	}
+
+	historyFile := path.Join(runner.HOME, "dsq_history")
+	l, err := readline.NewEx(&readline.Config{
+		Prompt:              "dsq> ",
+		HistoryFile:         historyFile,
+		InterruptPrompt:     "^D",
+		EOFPrompt:           "exit",
+		HistorySearchFold:   true,
+		FuncFilterInputRune: filterInput,
+		AutoComplete:        completer,
+	})
+	if err != nil {
+		return err
+	}
+
+	defer l.Close()
+
+	for {
+		queryRaw, err := l.Readline()
+		if err != nil {
+			return err
+		}
+
+		queryRaw = strings.TrimSpace(queryRaw)
+		if queryRaw == "" {
+			continue
+		}
+
+		if queryRaw == "exit" {
+			// prints bye like mysql
+			fmt.Println("bye")
+			return nil
+		}
+
+		err = runQuery(queryRaw, project, ec, args, files)
+		if err != nil {
+			return err
+		}
+	}
+}
+
 type args struct {
 	pipedMimetype string
 	pretty        bool
@@ -227,6 +334,7 @@ type args struct {
 	cacheSettings runner.CacheSettings
 	nonFlagArgs   []string
 	dumpCacheFile bool
+	isInteractive bool
 }
 
 func getArgs() (*args, error) {
@@ -294,6 +402,13 @@ func getArgs() (*args, error) {
 			continue
 		}
 
+		if arg == "--interactive" || arg == "-i" {
+			args.isInteractive = true
+			args.pretty = true
+			args.cacheSettings.Enabled = true
+			continue
+		}
+
 		args.nonFlagArgs = append(args.nonFlagArgs, arg)
 	}
 
@@ -451,7 +566,7 @@ func _main() error {
 	}
 
 	// No query, just dump transformed file directly out
-	if lastNonFlagArg == "" {
+	if lastNonFlagArg == "" && !args.isInteractive {
 		resultFile := ec.GetPanelResultsFile(project.Id, project.Pages[0].Panels[0].Id)
 		return dumpJSONFile(resultFile, args.pretty, args.schema)
 	}
@@ -466,41 +581,11 @@ func _main() error {
 	}
 	project.Connectors = append(project.Connectors, *connector)
 
-	query := rewriteQuery(lastNonFlagArg)
-	panel := &runner.PanelInfo{
-		Type:    runner.DatabasePanel,
-		Content: query,
-		Id:      uuid.New().String(),
-		Name:    uuid.New().String(),
-		DatabasePanelInfo: &runner.DatabasePanelInfo{
-			Database: runner.DatabasePanelInfoDatabase{
-				ConnectorId: connector.Id,
-			},
-		},
+	if args.isInteractive {
+		return repl(project, &ec, args, files)
 	}
 
-	err = ec.EvalDatabasePanel(project, 0, panel, nil, args.cacheSettings)
-	if err != nil {
-		if e, ok := err.(*runner.DSError); ok && e.Name == "NotAnArrayOfObjectsError" {
-			rest := "."
-			nth, err := strconv.Atoi(e.TargetPanelId)
-			if err == nil {
-				rest = ": " + files[nth] + "."
-			}
-			return fmt.Errorf("Input is not an array of objects%s\n", rest)
-		}
-
-		if e, ok := err.(*runner.DSError); ok && e.Name == "UserError" {
-			if e.Message[len(e.Message)-1] != '.' {
-				e.Message += "."
-			}
-			return errors.New(e.Message)
-		}
-		return err
-	}
-
-	resultFile := ec.GetPanelResultsFile(project.Id, panel.Id)
-	return dumpJSONFile(resultFile, args.pretty, args.schema)
+	return runQuery(lastNonFlagArg, project, &ec, args, files)
 }
 
 func main() {
